%--------------------------------- RELOAD --------------------------------------
%
%    Purpose: Reload the data into an existing set from a document
%             (possibly created by UNLOAD).
%
%    Syntax   RELOAD <set name> <output document> [<directory name>]
%
%    Notes:   1) If the parameters are not given you will be prompted.
%             2) <set name> must be a entity set or relationship with fields.
%             3) <output document> - only the first 8 characters are used.
%
%------------------------------------------------------------------------------

on break
	set restore
	release $utils
	return
endon

if '#<1>' = '?'
	output
	output 'RELOAD imports records from a document into an entity set (or'
	output 'relationship with fields). Typically, the document is a structured'
	output 'document produced by using the UNLOAD utility program.'
	output
	output
	output "use: RELOAD [ <entity set>    [<output document>  [<directory name>]]]"
	output "            [ <relationship>                                         ]"
	pause
	return
endif

set save
set reset

% Access the ZIM utilities directory, $utils, if it isn't already accessed.

set errors off
set warnings off
let $lasterrcode = 0  $lasterrlevel = 0
let $utils._dirstring = ""
if $lasterrcode = 4008
   let $lasterrcode = 0  $lasterrlevel = 0
   access $utils read
endif
set errors on
set warnings on

% Terminate if unable to access $utils
if $lasterrlevel > 2	% Error accessing $utils...
	output
	output "*** ERROR *** Unable to access ZIM utilities directory, '$Utils'."
	output "*** ERROR *** Check that your ZIM environment variable is set up properly"
	output
	set exception break
endif

set information off

output '                    *** RELOAD ***'
output
output 'To abort at any time simply press ENTER when prompted for information.'
output
output
if '#<1>' = ''
    output '* Enter the name of the entity set or relationship to be reloaded.'
    output 'Entity Set or Relationship name: ';
    input <1>
    output
    if '#<1>' = ''
        set exception break
    endif
	if '#<3>' = ''
		output '* Enter the name of the ZIM directory in which (#<1>) is defined.'
		output '  (Press return for the default directory "zim".)'
		output 'ZIM Directory name: ';
    	input <3>
    	output
    	if '#<3>' = ''
    	    let <3> = 'zim'
		endif
	endif
else
    let <8> = $true		% Flag to indicate parameters passed on call to RELOAD.
    if '#<3>' = ''
        let <3> = 'zim'
    endif
endif

    % Data Dictionary search for the entity set or relationship name.
    % Note: ent and rel with same name ==> everything ok but messages.

let <7> = "#<3>"
find 1 entitysets where entname='#<1>' and dirname = "#<3>"
if $setcount = 0
	find 1 entitysets where entname='#<1>' and dirname = ""
	let <7> = ""
endif
if $setcount <> 0
    let <1> = $trim(EntName)
    let <9> = 'entity set'
else
	let <7> = "#<3>"
    find 1 relationships where relname='#<1>' and dirname = "#<3>"
	if $setcount = 0
	    find 1 relationships where relname='#<1>' and dirname = ""
		let <7> = ""
	endif
    if $setcount <> 0
        let <1> = $trim (RelName)
        let <9> = 'relationship'
    else
        output '+++ RELOAD error +++ No entity set or relationship definition for (#<1>)'
        output '                     in the Data Dictionary set ENTITYSETS or RELATIONSHIPS.'
        set exception break
    endif
endif
while
    if '#<2>'=''
        output '* Enter the name of the document in which (#<1>)'
        output '  data has been stored (max 8 characters).'
        output 'Document name: ';
        input <2>
        output
    endif
    if '#<2>' = ''
		set exception break
    endif
    let <2> = $substring ('#<2>', 1, 8)
            % Check if docname in-use.
    find 1 docs where docname='#<2>' and dirname = "#<7>"
    if $setcount = 0
        output '*** RELOAD Error *** The Document name (#<2>) is  unknown.'
		let <2> = ""
		continue
    endif
    let <4> = $trim(FileName)
	break
endwhile

output
output '----------------------------- RELOAD --------------------------------'
output $tochar('#<9>',18) $tochar('directory',18) $tochar('document',18) 'file'
output $tochar('#<1>',18) $tochar('#<3>',18)        $tochar('#<2>',18)       '#<4>'
output
output 'Does everything seem OK to start reloading? ( i.e. no error messages? )'
output 'Enter y/n:';
input <5>
if $tolower('#<5>') <> 'y'
    set exception break
endif
output
output 'Reloading (#<1>...)'
output
SET EOLNTRANSLATE $Utils.cEOLNTRANSLATE  %% in case data had CR/LF in it.
add #<3>.#<1> from #<3>.#<2>
if $errcode = 0            % Error or warning not detected.
    output '* The data in the document (#<2>) with filename (#<4>)'
    output '  has been reloaded into #<9> (#<1>).'
    output
endif
output 'Do you want the document (#<2>) removed?'
output 'Enter y/n:';
input <5>
if $tolower('#<5>') = 'y'    % Remove document on request.
    erase document #<3>.#<2> from #<3>
    delete all docs where docname='#<2>' and dirname = "#<7>"
    delete all fields where ownername='#<2>' and dirname = "#<7>"
    output '* The document (#<2>) has been removed.'
    output '  Note: the file (#<4>) has NOT been erased.'
endif
output
output 'Reload of #<9> (#<1>) complete.'
%pause
set exception break
